<?php

namespace Drupal\openstory;

use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormState;

/**
 * Provides the entity.
 */
class EntityStructure {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * Constructs a ContentEntityForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManager $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Get entity form display.
   *
   * @param string $entity_type
   *   The entity type.
   * @param string $bundle
   *   Entity bundle.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   Return the entity.
   */
  public function getEntityStorage($entity_type, $bundle) {
    switch ($entity_type) {
      case 'node':
        return $this->entityTypeManager->getStorage('node')->create(['type' => $bundle]);

      case 'taxonomy_term':
        return $this->entityTypeManager->getStorage('taxonomy_term')->create(['vid' => $bundle]);

      case 'user':
        return $this->entityTypeManager->getStorage('user')->create([
          'name' => 'serialization_test_user',
          'mail' => 'foo@example.com',
          'pass' => '123456',
        ]);

      case 'comment':
        $values = [];
        $definition = $this->entityTypeManager->getDefinition("comment");
        $bundle_key = $definition->get('entity_keys')['bundle'];
        $values[$bundle_key] = $bundle;
        return $this->entityTypeManager->getStorage('comment')->create($values);

      case 'contact_message':
        return $this->entityTypeManager
          ->getStorage('contact_message')
          ->create([
            'contact_form' => $bundle,
          ]);

      default:
    }
  }

  /**
   * Get entity form display.
   *
   * @param string $entity_type
   *   The entity type.
   * @param string $bundle
   *   Entity bundle.
   *
   * @return \Drupal\Core\Entity\Entity\EntityFormDisplay|null
   *   The bundle entity.
   */
  public function getEntityFormDisplay($entity_type, $bundle) {
    switch ($entity_type) {
      case 'node':
        $storageFormDisplay = $this->entityTypeManager->getStorage('entity_form_display')->load('node' . '.' . $bundle . '.' . 'default');
        if (empty($storageFormDisplay)) {
          $storageFormDisplay = $this->createEntityStorage('node', $bundle);
        }
        return $storageFormDisplay;

      case 'taxonomy_term':
        $storageFormDisplay = $this->entityTypeManager->getStorage('entity_form_display')->load('taxonomy_term' . '.' . $bundle . '.' . 'default');
        if (empty($storageFormDisplay)) {
          $storageFormDisplay = $this->createEntityStorage('taxonomy_term', $bundle);
        }
        return $storageFormDisplay;

      case 'user':
        $storageFormDisplay = $this->entityTypeManager->getStorage('entity_form_display')->load('user' . '.' . $bundle . '.' . 'default');
        if (empty($storageFormDisplay)) {
          $storageFormDisplay = $this->createEntityStorage('user', 'user');
        }
        return $storageFormDisplay;

      case 'comment':
        $storageFormDisplay = $this->entityTypeManager->getStorage('entity_form_display')->load('comment' . '.' . $bundle . '.' . 'default');
        if (empty($storageFormDisplay)) {
          $storageFormDisplay = $this->createEntityStorage('comment', $bundle);
        }
        return $storageFormDisplay;

      case 'contact_message':
        $storageFormDisplay = $this->entityTypeManager->getStorage('entity_form_display')->load('contact_message' . '.' . $bundle . '.' . 'default');
        if (empty($storageFormDisplay)) {
          $storageFormDisplay = $this->createEntityStorage('contact_message', $bundle);
        }
        return $storageFormDisplay;

      default:
    }
  }

  /**
   * Get form of entity type node.
   *
   * @param string $entity_type
   *   The entity type.
   * @param string $bundle
   *   Entity bundle.
   * @param array $form
   *   Entity form.
   * @param \Drupal\Core\Form\FormState $form_state
   *   Entity form state.
   *
   * @return array|null
   *   Entity form returned.
   */
  public function getFormEntity($entity_type, $bundle, array $form, FormState $form_state) {
    $storageFormDisplay = $this->getEntityFormDisplay($entity_type, $bundle);
    $entity = $this->getEntityStorage($entity_type, $bundle);
    if (!empty($storageFormDisplay)) {
      $storageFormDisplay->buildForm($entity, $form, $form_state);
    }
    return $form;
  }

  /**
   * Create entity storage.
   *
   * @param string $entity_type
   *   The entity type.
   * @param string $bundle
   *   Entity bundle.
   *
   * @return \Drupal\Core\Entity\Entity\EntityFormDisplay|null
   *   The bundle entity.
   */
  public function createEntityStorage($entity_type, $bundle) {
    $values = [
      'targetEntityType' => $entity_type,
      'bundle' => $bundle,
      'mode' => 'default',
      'status' => TRUE,
    ];
    $storage = $this->entityTypeManager->getStorage('entity_form_display')->create($values);
    return $storage;
  }

  /**
   * Create/Update file with entity structure.
   *
   * @param string $entity_type
   *   The entity type.
   * @param string $bundle
   *   Entity bundle.
   * @param array $form
   *   Entity form.
   */
  public function createFile($entity_type, $bundle, array $form) {
    $file = $entity_type . '.' . $bundle . '.json';
    $string_json = json_encode($form);
    $dir = "public://storeEntities";
    if (!is_dir($dir)) {
      mkdir($dir);
    }
    $destination = $dir . '/' . $file;
    file_save_data($string_json, $destination, FILE_EXISTS_REPLACE);
  }

  /**
   * Check if entity file exists.
   *
   * @param string $entity_type
   *   The entity type.
   * @param string $bundle
   *   Entity bundle.
   *
   * @return string|null
   *   File name.
   */
  public function checkFileExists($entity_type, $bundle) {
    $dir = "public://storeEntities";
    $fileName = $entity_type . '.' . $bundle . '.json';
    $file = $dir . '/' . $fileName;
    if (!file_exists($file)) {
      return NULL;
    }
    return $file;
  }

  /**
   * Check if entity file exists.
   *
   * @param string $filePath
   *   The entity type.
   *
   * @return array
   *   File content.
   */
  public function getEntityFile($filePath) {
    $fileContent = file_get_contents($filePath, TRUE);
    $data = json_decode($fileContent, TRUE);
    return $data;
  }

  /**
   * Check if user has permission.
   *
   * @param string $entity_type
   *   The entity type.
   * @param string $bundle
   *   The entity bundle.
   *
   * @return string
   *   Permission name.
   */
  public function getUserPermissionName($entity_type, $bundle) {
    switch ($entity_type) {
      case 'node':
        $permissionName = 'create ' . $bundle . ' content';
        break;

      case 'taxonomy_term':
        $permissionName = 'edit terms in ' . $bundle;
        break;

      case 'user':
        $permissionName = 'administer users';
        break;

      case 'contact_message':
        $permissionName = 'administer contact forms';
        break;

      case 'comment':
        $permissionName = 'post comments';
        break;

      default:

    }
    return $permissionName;
  }

}
